@@ -76,8 +76,8 @@ struct _CtkWindow
 
     ParsedAttribute        *attribute_list;
 
-    GtkTreeIter             iter;
-    GtkWidget              *widget;
+    GtkTextTagTable        *help_tag_table;
+    GtkTextBuffer          *help_text_buffer;
 };
 
 struct _CtkWindowClass
